/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Add a {@code SparseTensor} to a {@code SparseTensorsMap} return its handle.
 * <p>
 * A {@code SparseTensor} is represented by three tensors: {@code sparse_indices},
 * {@code sparse_values}, and {@code sparse_shape}.
 * <p>
 * This operator takes the given {@code SparseTensor} and adds it to a container
 * object (a {@code SparseTensorsMap}).  A unique key within this container is generated
 * in the form of an {@code int64}, and this is the value that is returned.
 * <p>
 * The {@code SparseTensor} can then be read out as part of a minibatch by passing
 * the key as a vector element to {@code TakeManySparseFromTensorsMap}.  To ensure
 * the correct {@code SparseTensorsMap} is accessed, ensure that the same
 * {@code container} and {@code shared_name} are passed to that Op.  If no {@code shared_name}
 * is provided here, instead use the <i>name</i> of the Operation created by calling
 * {@code AddSparseToTensorsMap} as the {@code shared_name} passed to
 * {@code TakeManySparseFromTensorsMap}.  Ensure the Operations are colocated.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class AddSparseToTensorsMap extends PrimitiveOp implements Operand<Long> {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.AddSparseToTensorsMap}
   */
  public static class Options {
    
    /**
     * @param container The container name for the {@code SparseTensorsMap} created by this op.
     */
    public Options container(String container) {
      this.container = container;
      return this;
    }
    
    /**
     * @param sharedName The shared name for the {@code SparseTensorsMap} created by this op.
     * If blank, the new Operation's unique name is used.
     */
    public Options sharedName(String sharedName) {
      this.sharedName = sharedName;
      return this;
    }
    
    private String container;
    private String sharedName;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new AddSparseToTensorsMap operation to the graph.
   * 
   * @param scope current graph scope
   * @param sparseIndices 2-D.  The {@code indices} of the {@code SparseTensor}.
   * @param sparseValues 1-D.  The {@code values} of the {@code SparseTensor}.
   * @param sparseShape 1-D.  The {@code shape} of the {@code SparseTensor}.
   * @param options carries optional attributes values
   * @return a new instance of AddSparseToTensorsMap
   */
  public static <T> AddSparseToTensorsMap create(Scope scope, Operand<Long> sparseIndices, Operand<T> sparseValues, Operand<Long> sparseShape, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("AddSparseToTensorsMap", scope.makeOpName("AddSparseToTensorsMap"));
    opBuilder.addInput(sparseIndices.asOutput());
    opBuilder.addInput(sparseValues.asOutput());
    opBuilder.addInput(sparseShape.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.container != null) {
          opBuilder.setAttr("container", opts.container);
        }
        if (opts.sharedName != null) {
          opBuilder.setAttr("shared_name", opts.sharedName);
        }
      }
    }
    return new AddSparseToTensorsMap(opBuilder.build());
  }
  
  /**
   * @param container The container name for the {@code SparseTensorsMap} created by this op.
   */
  public static Options container(String container) {
    return new Options().container(container);
  }
  
  /**
   * @param sharedName The shared name for the {@code SparseTensorsMap} created by this op.
   * If blank, the new Operation's unique name is used.
   */
  public static Options sharedName(String sharedName) {
    return new Options().sharedName(sharedName);
  }
  
  /**
   * 0-D.  The handle of the {@code SparseTensor} now stored in the
   * {@code SparseTensorsMap}.
   */
  public Output<Long> sparseHandle() {
    return sparseHandle;
  }
  
  @Override
  public Output<Long> asOutput() {
    return sparseHandle;
  }
  
  private Output<Long> sparseHandle;
  
  private AddSparseToTensorsMap(Operation operation) {
    super(operation);
    int outputIdx = 0;
    sparseHandle = operation.output(outputIdx++);
  }
}
