/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Adjust the contrast of one or more images.
 * <p>
 * {@code images} is a tensor of at least 3 dimensions.  The last 3 dimensions are
 * interpreted as {@code [height, width, channels]}.  The other dimensions only
 * represent a collection of images, such as {@code [batch, height, width, channels].}
 * <p>
 * Contrast is adjusted independently for each channel of each image.
 * <p>
 * For each channel, the Op first computes the mean of the image pixels in the
 * channel and then adjusts each component of each pixel to
 * {@code (x - mean) * contrast_factor + mean}.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class AdjustContrast extends PrimitiveOp implements Operand<Float> {
  
  /**
   * Factory method to create a class to wrap a new AdjustContrast operation to the graph.
   * 
   * @param scope current graph scope
   * @param images Images to adjust.  At least 3-D.
   * @param contrastFactor A float multiplier for adjusting contrast.
   * @return a new instance of AdjustContrast
   */
  public static AdjustContrast create(Scope scope, Operand<Float> images, Operand<Float> contrastFactor) {
    OperationBuilder opBuilder = scope.graph().opBuilder("AdjustContrastv2", scope.makeOpName("AdjustContrast"));
    opBuilder.addInput(images.asOutput());
    opBuilder.addInput(contrastFactor.asOutput());
    return new AdjustContrast(opBuilder.build());
  }
  
  /**
   * The contrast-adjusted image or images.
   */
  public Output<Float> output() {
    return output;
  }
  
  @Override
  public Output<Float> asOutput() {
    return output;
  }
  
  private Output<Float> output;
  
  private AdjustContrast(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
