/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Adjust the hue of one or more images.
 * <p>
 * {@code images} is a tensor of at least 3 dimensions.  The last dimension is
 * interpretted as channels, and must be three.
 * <p>
 * The input image is considered in the RGB colorspace. Conceptually, the RGB
 * colors are first mapped into HSV. A delta is then applied all the hue values,
 * and then remapped back to RGB colorspace.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class AdjustHue extends PrimitiveOp implements Operand<Float> {
  
  /**
   * Factory method to create a class to wrap a new AdjustHue operation to the graph.
   * 
   * @param scope current graph scope
   * @param images Images to adjust.  At least 3-D.
   * @param delta A float delta to add to the hue.
   * @return a new instance of AdjustHue
   */
  public static AdjustHue create(Scope scope, Operand<Float> images, Operand<Float> delta) {
    OperationBuilder opBuilder = scope.graph().opBuilder("AdjustHue", scope.makeOpName("AdjustHue"));
    opBuilder.addInput(images.asOutput());
    opBuilder.addInput(delta.asOutput());
    return new AdjustHue(opBuilder.build());
  }
  
  /**
   * The hue-adjusted image or images.
   */
  public Output<Float> output() {
    return output;
  }
  
  @Override
  public Output<Float> asOutput() {
    return output;
  }
  
  private Output<Float> output;
  
  private AdjustHue(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
