/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Counts the number of occurrences of each value in an integer array.
 * <p>
 * Outputs a vector with length {@code size} and the same dtype as {@code weights}. If
 * {@code weights} are empty, then index {@code i} stores the number of times the value {@code i} is
 * counted in {@code arr}. If {@code weights} are non-empty, then index {@code i} stores the sum of
 * the value in {@code weights} at each index where the corresponding value in {@code arr} is
 * {@code i}.
 * <p>
 * Values in {@code arr} outside of the range [0, size) are ignored.
 * 
 * @param <T> data type for {@code bins()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Bincount<T extends Number> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new Bincount operation to the graph.
   * 
   * @param scope current graph scope
   * @param arr int32 {@code Tensor}.
   * @param size non-negative int32 scalar {@code Tensor}.
   * @param weights is an int32, int64, float32, or float64 {@code Tensor} with the same
   * shape as {@code arr}, or a length-0 {@code Tensor}, in which case it acts as all weights
   * equal to 1.
   * @return a new instance of Bincount
   */
  public static <T extends Number> Bincount<T> create(Scope scope, Operand<Integer> arr, Operand<Integer> size, Operand<T> weights) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Bincount", scope.makeOpName("Bincount"));
    opBuilder.addInput(arr.asOutput());
    opBuilder.addInput(size.asOutput());
    opBuilder.addInput(weights.asOutput());
    return new Bincount<T>(opBuilder.build());
  }
  
  /**
   * 1D {@code Tensor} with length equal to {@code size}. The counts or summed weights for
   * each value in the range [0, size).
   */
  public Output<T> bins() {
    return bins;
  }
  
  @Override
  public Output<T> asOutput() {
    return bins;
  }
  
  private Output<T> bins;
  
  private Bincount(Operation operation) {
    super(operation);
    int outputIdx = 0;
    bins = operation.output(outputIdx++);
  }
}
