/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Returns the complex conjugate of a complex number.
 * <p>
 * Given a tensor {@code input} of complex numbers, this operation returns a tensor of
 * complex numbers that are the complex conjugate of each element in {@code input}. The
 * complex numbers in {@code input} must be of the form \\(a + bj\\), where <i>a</i> is the
 * real part and <i>b</i> is the imaginary part.
 * <p>
 * The complex conjugate returned by this operation is of the form \\(a - bj\\).
 * <p>
 * For example:
 * <pre>{@code
 * # tensor 'input' is [-2.25 + 4.75j, 3.25 + 5.75j]
 * tf.conj(input) ==> [-2.25 - 4.75j, 3.25 - 5.75j]
 * }</pre>
 * 
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Conj<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new Conj operation to the graph.
   * 
   * @param scope current graph scope
   * @param input 
   * @return a new instance of Conj
   */
  public static <T> Conj<T> create(Scope scope, Operand<T> input) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Conj", scope.makeOpName("Conj"));
    opBuilder.addInput(input.asOutput());
    return new Conj<T>(opBuilder.build());
  }
  
  /**
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private Conj(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
