/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the gradient of the crop_and_resize op wrt the input boxes tensor.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class CropAndResizeGradBoxes extends PrimitiveOp implements Operand<Float> {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.CropAndResizeGradBoxes}
   */
  public static class Options {
    
    /**
     * @param method A string specifying the interpolation method. Only 'bilinear' is
     * supported for now.
     */
    public Options method(String method) {
      this.method = method;
      return this;
    }
    
    private String method;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new CropAndResizeGradBoxes operation to the graph.
   * 
   * @param scope current graph scope
   * @param grads A 4-D tensor of shape {@code [num_boxes, crop_height, crop_width, depth]}.
   * @param image A 4-D tensor of shape {@code [batch, image_height, image_width, depth]}.
   * Both {@code image_height} and {@code image_width} need to be positive.
   * @param boxes A 2-D tensor of shape {@code [num_boxes, 4]}. The {@code i}-th row of the tensor
   * specifies the coordinates of a box in the {@code box_ind[i]} image and is specified
   * in normalized coordinates {@code [y1, x1, y2, x2]}. A normalized coordinate value of
   * {@code y} is mapped to the image coordinate at {@code y * (image_height - 1)}, so as the
   * {@code [0, 1]} interval of normalized image height is mapped to
   * {@code [0, image_height - 1] in image height coordinates. We do allow y1 > y2, in
   * which case the sampled crop is an up-down flipped version of the original
   * image. The width dimension is treated similarly. Normalized coordinates
   * outside the }[0, 1]{@code  range are allowed, in which case we use
   * }extrapolation_value` to extrapolate the input image values.
   * @param boxInd A 1-D tensor of shape {@code [num_boxes]} with int32 values in {@code [0, batch)}.
   * The value of {@code box_ind[i]} specifies the image that the {@code i}-th box refers to.
   * @param options carries optional attributes values
   * @return a new instance of CropAndResizeGradBoxes
   */
  public static <T extends Number> CropAndResizeGradBoxes create(Scope scope, Operand<Float> grads, Operand<T> image, Operand<Float> boxes, Operand<Integer> boxInd, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("CropAndResizeGradBoxes", scope.makeOpName("CropAndResizeGradBoxes"));
    opBuilder.addInput(grads.asOutput());
    opBuilder.addInput(image.asOutput());
    opBuilder.addInput(boxes.asOutput());
    opBuilder.addInput(boxInd.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.method != null) {
          opBuilder.setAttr("method", opts.method);
        }
      }
    }
    return new CropAndResizeGradBoxes(opBuilder.build());
  }
  
  /**
   * @param method A string specifying the interpolation method. Only 'bilinear' is
   * supported for now.
   */
  public static Options method(String method) {
    return new Options().method(method);
  }
  
  /**
   * A 2-D tensor of shape {@code [num_boxes, 4]}.
   */
  public Output<Float> output() {
    return output;
  }
  
  @Override
  public Output<Float> asOutput() {
    return output;
  }
  
  private Output<Float> output;
  
  private CropAndResizeGradBoxes(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
