/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Applies set operation along last dimension of {@code Tensor} and {@code SparseTensor}.
 * <p>
 * See SetOperationOp::SetOperationFromContext for values of {@code set_operation}.
 * <p>
 * Input {@code set2} is a {@code SparseTensor} represented by {@code set2_indices}, {@code set2_values},
 * and {@code set2_shape}. For {@code set2} ranked {@code n}, 1st {@code n-1} dimensions must be the same
 * as {@code set1}. Dimension {@code n} contains values in a set, duplicates are allowed but
 * ignored.
 * <p>
 * If {@code validate_indices} is {@code True}, this op validates the order and range of {@code set2}
 * indices.
 * <p>
 * Output {@code result} is a {@code SparseTensor} represented by {@code result_indices},
 * {@code result_values}, and {@code result_shape}. For {@code set1} and {@code set2} ranked {@code n}, this
 * has rank {@code n} and the same 1st {@code n-1} dimensions as {@code set1} and {@code set2}. The {@code nth}
 * dimension contains the result of {@code set_operation} applied to the corresponding
 * {@code [0...n-1]} dimension of {@code set}.
 * 
 * @param <T> data type for {@code resultValues()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class DenseToSparseSetOperation<T> extends PrimitiveOp {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.DenseToSparseSetOperation}
   */
  public static class Options {
    
    /**
     * @param validateIndices 
     */
    public Options validateIndices(Boolean validateIndices) {
      this.validateIndices = validateIndices;
      return this;
    }
    
    private Boolean validateIndices;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new DenseToSparseSetOperation operation to the graph.
   * 
   * @param scope current graph scope
   * @param set1 {@code Tensor} with rank {@code n}. 1st {@code n-1} dimensions must be the same as {@code set2}.
   * Dimension {@code n} contains values in a set, duplicates are allowed but ignored.
   * @param set2Indices 2D {@code Tensor}, indices of a {@code SparseTensor}. Must be in row-major
   * order.
   * @param set2Values 1D {@code Tensor}, values of a {@code SparseTensor}. Must be in row-major
   * order.
   * @param set2Shape 1D {@code Tensor}, shape of a {@code SparseTensor}. {@code set2_shape[0...n-1]} must
   * be the same as the 1st {@code n-1} dimensions of {@code set1}, {@code result_shape[n]} is the
   * max set size across {@code n-1} dimensions.
   * @param setOperation 
   * @param options carries optional attributes values
   * @return a new instance of DenseToSparseSetOperation
   */
  public static <T> DenseToSparseSetOperation<T> create(Scope scope, Operand<T> set1, Operand<Long> set2Indices, Operand<T> set2Values, Operand<Long> set2Shape, String setOperation, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("DenseToSparseSetOperation", scope.makeOpName("DenseToSparseSetOperation"));
    opBuilder.addInput(set1.asOutput());
    opBuilder.addInput(set2Indices.asOutput());
    opBuilder.addInput(set2Values.asOutput());
    opBuilder.addInput(set2Shape.asOutput());
    opBuilder.setAttr("set_operation", setOperation);
    if (options != null) {
      for (Options opts : options) {
        if (opts.validateIndices != null) {
          opBuilder.setAttr("validate_indices", opts.validateIndices);
        }
      }
    }
    return new DenseToSparseSetOperation<T>(opBuilder.build());
  }
  
  /**
   * @param validateIndices 
   */
  public static Options validateIndices(Boolean validateIndices) {
    return new Options().validateIndices(validateIndices);
  }
  
  /**
   * 2D indices of a {@code SparseTensor}.
   */
  public Output<Long> resultIndices() {
    return resultIndices;
  }
  
  /**
   * 1D values of a {@code SparseTensor}.
   */
  public Output<T> resultValues() {
    return resultValues;
  }
  
  /**
   * 1D {@code Tensor} shape of a {@code SparseTensor}. {@code result_shape[0...n-1]} is
   * the same as the 1st {@code n-1} dimensions of {@code set1} and {@code set2}, {@code result_shape[n]}
   * is the max result set size across all {@code 0...n-1} dimensions.
   */
  public Output<Long> resultShape() {
    return resultShape;
  }
  
  private Output<Long> resultIndices;
  private Output<T> resultValues;
  private Output<Long> resultShape;
  
  private DenseToSparseSetOperation(Operation operation) {
    super(operation);
    int outputIdx = 0;
    resultIndices = operation.output(outputIdx++);
    resultValues = operation.output(outputIdx++);
    resultShape = operation.output(outputIdx++);
  }
}
