/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;

/**
 * Exits the current frame to its parent frame.
 * <p>
 * Exit makes its input {@code data} available to the parent frame.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
public final class Exit<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new Exit operation to the graph.
   * 
   * @param scope current graph scope
   * @param data The tensor to be made available to the parent frame.
   * @return a new instance of Exit
   */
  public static <T> Exit<T> create(Scope scope, Operand<T> data) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Exit", scope.makeOpName("Exit"));
    opBuilder.addInput(data.asOutput());
    return new Exit<T>(opBuilder.build());
  }
  
  /**
   * The same tensor as {@code data}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private Exit(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
