/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * 3D fast Fourier transform.
 * <p>
 * Computes the 3-dimensional discrete Fourier transform over the inner-most 3
 * dimensions of {@code input}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class FFT3D<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new FFT3D operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A complex64 tensor.
   * @return a new instance of FFT3D
   */
  public static <T> FFT3D<T> create(Scope scope, Operand<T> input) {
    OperationBuilder opBuilder = scope.graph().opBuilder("FFT3D", scope.makeOpName("FFT3D"));
    opBuilder.addInput(input.asOutput());
    return new FFT3D<T>(opBuilder.build());
  }
  
  /**
   * A complex64 tensor of the same shape as {@code input}. The inner-most 3
   *   dimensions of {@code input} are replaced with their 3D Fourier transform.
   * <p>
   * @compatibility(numpy)
   * Equivalent to np.fft.fftn with 3 dimensions.
   * @end_compatibility
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private FFT3D(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
