/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Fake-quantize the 'inputs' tensor of type float via global float scalars {@code min}
 * <p>
 * and {@code max} to 'outputs' tensor of same shape as {@code inputs}.
 * <p>
 * {@code [min; max]} define the clamping range for the {@code inputs} data.
 * {@code inputs} values are quantized into the quantization range ({@code [0; 2^num_bits - 1]}
 * when {@code narrow_range} is false and {@code [1; 2^num_bits - 1]} when it is true) and
 * then de-quantized and output as floats in {@code [min; max]} interval.
 * {@code num_bits} is the bitwidth of the quantization; between 2 and 16, inclusive.
 * <p>
 * This operation has a gradient and thus allows for training {@code min} and {@code max}
 * values.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class FakeQuantWithMinMaxVars extends PrimitiveOp implements Operand<Float> {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.FakeQuantWithMinMaxVars}
   */
  public static class Options {
    
    /**
     * @param numBits 
     */
    public Options numBits(Long numBits) {
      this.numBits = numBits;
      return this;
    }
    
    /**
     * @param narrowRange 
     */
    public Options narrowRange(Boolean narrowRange) {
      this.narrowRange = narrowRange;
      return this;
    }
    
    private Long numBits;
    private Boolean narrowRange;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new FakeQuantWithMinMaxVars operation to the graph.
   * 
   * @param scope current graph scope
   * @param inputs 
   * @param min 
   * @param max 
   * @param options carries optional attributes values
   * @return a new instance of FakeQuantWithMinMaxVars
   */
  public static FakeQuantWithMinMaxVars create(Scope scope, Operand<Float> inputs, Operand<Float> min, Operand<Float> max, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("FakeQuantWithMinMaxVars", scope.makeOpName("FakeQuantWithMinMaxVars"));
    opBuilder.addInput(inputs.asOutput());
    opBuilder.addInput(min.asOutput());
    opBuilder.addInput(max.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.numBits != null) {
          opBuilder.setAttr("num_bits", opts.numBits);
        }
        if (opts.narrowRange != null) {
          opBuilder.setAttr("narrow_range", opts.narrowRange);
        }
      }
    }
    return new FakeQuantWithMinMaxVars(opBuilder.build());
  }
  
  /**
   * @param numBits 
   */
  public static Options numBits(Long numBits) {
    return new Options().numBits(numBits);
  }
  
  /**
   * @param narrowRange 
   */
  public static Options narrowRange(Boolean narrowRange) {
    return new Options().narrowRange(narrowRange);
  }
  
  /**
   */
  public Output<Float> outputs() {
    return outputs;
  }
  
  @Override
  public Output<Float> asOutput() {
    return outputs;
  }
  
  private Output<Float> outputs;
  
  private FakeQuantWithMinMaxVars(Operation operation) {
    super(operation);
    int outputIdx = 0;
    outputs = operation.output(outputIdx++);
  }
}
