/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Convert one or more images from HSV to RGB.
 * <p>
 * Outputs a tensor of the same shape as the {@code images} tensor, containing the RGB
 * value of the pixels. The output is only well defined if the value in {@code images}
 * are in {@code [0,1]}.
 * <p>
 * See {@code rgb_to_hsv} for a description of the HSV encoding.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class HSVToRGB<T extends Number> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new HSVToRGB operation to the graph.
   * 
   * @param scope current graph scope
   * @param images 1-D or higher rank. HSV data to convert. Last dimension must be size 3.
   * @return a new instance of HSVToRGB
   */
  public static <T extends Number> HSVToRGB<T> create(Scope scope, Operand<T> images) {
    OperationBuilder opBuilder = scope.graph().opBuilder("HSVToRGB", scope.makeOpName("HSVToRGB"));
    opBuilder.addInput(images.asOutput());
    return new HSVToRGB<T>(opBuilder.build());
  }
  
  /**
   * {@code images} converted to RGB.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private HSVToRGB(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
