/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Outputs a {@code Summary} protocol buffer with a histogram.
 * <p>
 * The generated
 * <a href="https://www.tensorflow.org/code/tensorflow/core/framework/summary.proto">{@code Summary}</a>
 * has one summary value containing a histogram for {@code values}.
 * <p>
 * This op reports an {@code InvalidArgument} error if any value is not finite.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class HistogramSummary extends PrimitiveOp implements Operand<String> {
  
  /**
   * Factory method to create a class to wrap a new HistogramSummary operation to the graph.
   * 
   * @param scope current graph scope
   * @param tag Scalar.  Tag to use for the {@code Summary.Value}.
   * @param values Any shape. Values to use to build the histogram.
   * @return a new instance of HistogramSummary
   */
  public static <T extends Number> HistogramSummary create(Scope scope, Operand<String> tag, Operand<T> values) {
    OperationBuilder opBuilder = scope.graph().opBuilder("HistogramSummary", scope.makeOpName("HistogramSummary"));
    opBuilder.addInput(tag.asOutput());
    opBuilder.addInput(values.asOutput());
    return new HistogramSummary(opBuilder.build());
  }
  
  /**
   * Scalar. Serialized {@code Summary} protocol buffer.
   */
  public Output<String> summary() {
    return summary;
  }
  
  @Override
  public Output<String> asOutput() {
    return summary;
  }
  
  private Output<String> summary;
  
  private HistogramSummary(Operation operation) {
    super(operation);
    int outputIdx = 0;
    summary = operation.output(outputIdx++);
  }
}
