/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Inverse 3D real-valued fast Fourier transform.
 * <p>
 * Computes the inverse 3-dimensional discrete Fourier transform of a real-valued
 * signal over the inner-most 3 dimensions of {@code input}.
 * <p>
 * The inner-most 3 dimensions of {@code input} are assumed to be the result of {@code RFFT3D}:
 * The inner-most dimension contains the {@code fft_length / 2 + 1} unique components of
 * the DFT of a real-valued signal. If {@code fft_length} is not provided, it is computed
 * from the size of the inner-most 3 dimensions of {@code input}. If the FFT length used
 * to compute {@code input} is odd, it should be provided since it cannot be inferred
 * properly.
 * <p>
 * Along each axis {@code IRFFT3D} is computed on, if {@code fft_length} (or
 * {@code fft_length / 2 + 1} for the inner-most dimension) is smaller than the
 * corresponding dimension of {@code input}, the dimension is cropped. If it is larger,
 * the dimension is padded with zeros.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class IRFFT3D extends PrimitiveOp implements Operand<Float> {
  
  /**
   * Factory method to create a class to wrap a new IRFFT3D operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A complex64 tensor.
   * @param fftLength An int32 tensor of shape [3]. The FFT length for each dimension.
   * @return a new instance of IRFFT3D
   */
  public static IRFFT3D create(Scope scope, Operand<?> input, Operand<Integer> fftLength) {
    OperationBuilder opBuilder = scope.graph().opBuilder("IRFFT3D", scope.makeOpName("IRFFT3D"));
    opBuilder.addInput(input.asOutput());
    opBuilder.addInput(fftLength.asOutput());
    return new IRFFT3D(opBuilder.build());
  }
  
  /**
   * A float32 tensor of the same rank as {@code input}. The inner-most 3
   *   dimensions of {@code input} are replaced with the {@code fft_length} samples of their
   *   inverse 3D real Fourier transform.
   * <p>
   * @compatibility(numpy)
   * Equivalent to np.irfftn with 3 dimensions.
   * @end_compatibility
   */
  public Output<Float> output() {
    return output;
  }
  
  @Override
  public Output<Float> asOutput() {
    return output;
  }
  
  private Output<Float> output;
  
  private IRFFT3D(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
