/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Says whether the targets are in the top {@code K} predictions.
 * <p>
 * This outputs a {@code batch_size} bool array, an entry {@code out[i]} is {@code true} if the
 * prediction for the target class is among the top {@code k} predictions among
 * all predictions for example {@code i}. Note that the behavior of {@code InTopK} differs
 * from the {@code TopK} op in its handling of ties; if multiple classes have the
 * same prediction value and straddle the top-{@code k} boundary, all of those
 * classes are considered to be in the top {@code k}.
 * <p>
 * More formally, let
 * <p>
 *   \\(predictions_i\\) be the predictions for all classes for example {@code i},
 *   \\(targets_i\\) be the target class for example {@code i},
 *   \\(out_i\\) be the output for example {@code i},
 * <p>
 * $$out_i = predictions_{i, targets_i} \in TopKIncludingTies(predictions_i)$$
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class InTopK extends PrimitiveOp implements Operand<Boolean> {
  
  /**
   * Factory method to create a class to wrap a new InTopK operation to the graph.
   * 
   * @param scope current graph scope
   * @param predictions A {@code batch_size} x {@code classes} tensor.
   * @param targets A {@code batch_size} vector of class ids.
   * @param k Number of top elements to look at for computing precision.
   * @return a new instance of InTopK
   */
  public static <T extends Number> InTopK create(Scope scope, Operand<Float> predictions, Operand<T> targets, Long k) {
    OperationBuilder opBuilder = scope.graph().opBuilder("InTopK", scope.makeOpName("InTopK"));
    opBuilder.addInput(predictions.asOutput());
    opBuilder.addInput(targets.asOutput());
    opBuilder.setAttr("k", k);
    return new InTopK(opBuilder.build());
  }
  
  /**
   * Computed Precision at {@code k} as a {@code bool Tensor}.
   */
  public Output<Boolean> precision() {
    return precision;
  }
  
  @Override
  public Output<Boolean> asOutput() {
    return precision;
  }
  
  private Output<Boolean> precision;
  
  private InTopK(Operation operation) {
    super(operation);
    int outputIdx = 0;
    precision = operation.output(outputIdx++);
  }
}
