/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the matrix exponential of one or more square matrices:
 * <p>
 * exp(A) = \sum_{n=0}^\infty A^n/n!
 * <p>
 * The exponential is computed using a combination of the scaling and squaring
 * method and the Pade approximation. Details can be founds in:
 * Nicholas J. Higham, "The scaling and squaring method for the matrix exponential
 * revisited," SIAM J. Matrix Anal. Applic., 26:1179-1193, 2005.
 * <p>
 * The input is a tensor of shape {@code [..., M, M]} whose inner-most 2 dimensions
 * form square matrices. The output is a tensor of the same shape as the input
 * containing the exponential for all input submatrices {@code [..., :, :]}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class MatrixExponential<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new MatrixExponential operation to the graph.
   * 
   * @param scope current graph scope
   * @param input Shape is {@code [..., M, M]}.
   * @return a new instance of MatrixExponential
   */
  public static <T> MatrixExponential<T> create(Scope scope, Operand<T> input) {
    OperationBuilder opBuilder = scope.graph().opBuilder("MatrixExponential", scope.makeOpName("MatrixExponential"));
    opBuilder.addInput(input.asOutput());
    return new MatrixExponential<T>(opBuilder.build());
  }
  
  /**
   * Shape is {@code [..., M, M]}.
   * <p>
   * @compatibility(scipy)
   * Equivalent to scipy.linalg.expm
   * @end_compatibility
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private MatrixExponential(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
