/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Returns a batched matrix tensor with new batched diagonal values.
 * <p>
 * Given {@code input} and {@code diagonal}, this operation returns a tensor with the
 * same shape and values as {@code input}, except for the main diagonal of the
 * innermost matrices.  These will be overwritten by the values in {@code diagonal}.
 * <p>
 * The output is computed as follows:
 * <p>
 * Assume {@code input} has {@code k+1} dimensions {@code [I, J, K, ..., M, N]} and {@code diagonal} has
 * {@code k} dimensions {@code [I, J, K, ..., min(M, N)]}.  Then the output is a
 * tensor of rank {@code k+1} with dimensions {@code [I, J, K, ..., M, N]} where:
 * <p>
 *   * {@code output[i, j, k, ..., m, n] = diagonal[i, j, k, ..., n]} for {@code m == n}.
 *   * {@code output[i, j, k, ..., m, n] = input[i, j, k, ..., m, n]} for {@code m != n}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class MatrixSetDiag<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new MatrixSetDiag operation to the graph.
   * 
   * @param scope current graph scope
   * @param input Rank {@code k+1}, where {@code k >= 1}.
   * @param diagonal Rank {@code k}, where {@code k >= 1}.
   * @return a new instance of MatrixSetDiag
   */
  public static <T> MatrixSetDiag<T> create(Scope scope, Operand<T> input, Operand<T> diagonal) {
    OperationBuilder opBuilder = scope.graph().opBuilder("MatrixSetDiag", scope.makeOpName("MatrixSetDiag"));
    opBuilder.addInput(input.asOutput());
    opBuilder.addInput(diagonal.asOutput());
    return new MatrixSetDiag<T>(opBuilder.build());
  }
  
  /**
   * Rank {@code k+1}, with {@code output.shape = input.shape}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private MatrixSetDiag(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
