/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Forwards the value of an available tensor from {@code inputs} to {@code output}.
 * <p>
 * {@code Merge} waits for at least one of the tensors in {@code inputs} to become available.
 * It is usually combined with {@code Switch} to implement branching.
 * <p>
 * {@code Merge} forwards the first tensor to become available to {@code output}, and sets
 * {@code value_index} to its index in {@code inputs}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Merge<T> extends PrimitiveOp {
  
  /**
   * Factory method to create a class to wrap a new Merge operation to the graph.
   * 
   * @param scope current graph scope
   * @param inputs The input tensors, exactly one of which will become available.
   * @return a new instance of Merge
   */
  public static <T> Merge<T> create(Scope scope, Operand<T> inputs) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Merge", scope.makeOpName("Merge"));
    opBuilder.addInput(inputs.asOutput());
    return new Merge<T>(opBuilder.build());
  }
  
  /**
   * Will be set to the available input tensor.
   */
  public Output<T> output() {
    return output;
  }
  
  /**
   * The index of the chosen input tensor in {@code inputs}.
   */
  public Output<Integer> valueIndex() {
    return valueIndex;
  }
  
  private Output<T> output;
  private Output<Integer> valueIndex;
  
  private Merge(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
    valueIndex = operation.output(outputIdx++);
  }
}
