/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;
import org.tensorflow.types.UInt8;

/**
 * Computes element-wise population count (a.k.a. popcount, bitsum, bitcount).
 * <p>
 * For each entry in {@code x}, calculates the number of {@code 1} (on) bits in the binary
 * representation of that entry.
 * <p>
 * <b>NOTE</b>: It is more efficient to first {@code tf.bitcast} your tensors into
 * {@code int32} or {@code int64} and perform the bitcount on the result, than to feed in
 * 8- or 16-bit inputs and then aggregate the resulting counts.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class PopulationCount extends PrimitiveOp implements Operand<UInt8> {
  
  /**
   * Factory method to create a class to wrap a new PopulationCount operation to the graph.
   * 
   * @param scope current graph scope
   * @param x 
   * @return a new instance of PopulationCount
   */
  public static <T extends Number> PopulationCount create(Scope scope, Operand<T> x) {
    OperationBuilder opBuilder = scope.graph().opBuilder("PopulationCount", scope.makeOpName("PopulationCount"));
    opBuilder.addInput(x.asOutput());
    return new PopulationCount(opBuilder.build());
  }
  
  /**
   */
  public Output<UInt8> y() {
    return y;
  }
  
  @Override
  public Output<UInt8> asOutput() {
    return y;
  }
  
  private Output<UInt8> y;
  
  private PopulationCount(Operation operation) {
    super(operation);
    int outputIdx = 0;
    y = operation.output(outputIdx++);
  }
}
