/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * 3D real-valued fast Fourier transform.
 * <p>
 * Computes the 3-dimensional discrete Fourier transform of a real-valued signal
 * over the inner-most 3 dimensions of {@code input}.
 * <p>
 * Since the DFT of a real signal is Hermitian-symmetric, {@code RFFT3D} only returns the
 * {@code fft_length / 2 + 1} unique components of the FFT for the inner-most dimension
 * of {@code output}: the zero-frequency term, followed by the {@code fft_length / 2}
 * positive-frequency terms.
 * <p>
 * Along each axis {@code RFFT3D} is computed on, if {@code fft_length} is smaller than the
 * corresponding dimension of {@code input}, the dimension is cropped. If it is larger,
 * the dimension is padded with zeros.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class RFFT3D extends PrimitiveOp implements Operand<Object> {
  
  /**
   * Factory method to create a class to wrap a new RFFT3D operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A float32 tensor.
   * @param fftLength An int32 tensor of shape [3]. The FFT length for each dimension.
   * @return a new instance of RFFT3D
   */
  public static RFFT3D create(Scope scope, Operand<Float> input, Operand<Integer> fftLength) {
    OperationBuilder opBuilder = scope.graph().opBuilder("RFFT3D", scope.makeOpName("RFFT3D"));
    opBuilder.addInput(input.asOutput());
    opBuilder.addInput(fftLength.asOutput());
    return new RFFT3D(opBuilder.build());
  }
  
  /**
   * A complex64 tensor of the same rank as {@code input}. The inner-most 3
   *   dimensions of {@code input} are replaced with the their 3D Fourier transform. The
   *   inner-most dimension contains {@code fft_length / 2 + 1} unique frequency
   *   components.
   * <p>
   * @compatibility(numpy)
   * Equivalent to np.fft.rfftn with 3 dimensions.
   * @end_compatibility
   */
  public Output<?> output() {
    return output;
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public Output<Object> asOutput() {
    return (Output<Object>) output;
  }
  
  private Output<?> output;
  
  private RFFT3D(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
