/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the mean along segments of a tensor.
 * <p>
 * Read @{$math_ops#Segmentation$the section on segmentation} for an explanation of
 * segments.
 * <p>
 * Computes a tensor such that
 * \\(output_i = \frac{\sum_j data_j}{N}\\) where {@code mean} is
 * over {@code j} such that {@code segment_ids[j] == i} and {@code N} is the total number of
 * values summed.
 * <p>
 * If the mean is empty for a given segment ID {@code i}, {@code output[i] = 0}.
 * <p>
 * <div style="width:70%; margin:auto; margin-bottom:10px; margin-top:20px;">
 * <img style="width:100%" src="https://www.tensorflow.org/images/SegmentMean.png" alt>
 * </div>
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class SegmentMean<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new SegmentMean operation to the graph.
   * 
   * @param scope current graph scope
   * @param data 
   * @param segmentIds A 1-D tensor whose rank is equal to the rank of {@code data}'s
   * first dimension.  Values should be sorted and can be repeated.
   * @return a new instance of SegmentMean
   */
  public static <T, U extends Number> SegmentMean<T> create(Scope scope, Operand<T> data, Operand<U> segmentIds) {
    OperationBuilder opBuilder = scope.graph().opBuilder("SegmentMean", scope.makeOpName("SegmentMean"));
    opBuilder.addInput(data.asOutput());
    opBuilder.addInput(segmentIds.asOutput());
    return new SegmentMean<T>(opBuilder.build());
  }
  
  /**
   * Has same shape as data, except for dimension 0 which
   * has size {@code k}, the number of segments.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private SegmentMean(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
