/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * The gradient of SparseFillEmptyRows.
 * <p>
 * Takes vectors reverse_index_map, shaped {@code [N]}, and grad_values,
 * shaped {@code [N_full]}, where {@code N_full >= N} and copies data into either
 * {@code d_values} or {@code d_default_value}.  Here {@code d_values} is shaped {@code [N]} and
 * {@code d_default_value} is a scalar.
 * <p>
 *   d_values[j] = grad_values[reverse_index_map[j]]
 *   d_default_value = sum_{k : 0 .. N_full - 1} (
 *      grad_values[k] * 1{k not in reverse_index_map})
 * 
 * @param <T> data type for {@code dValues()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class SparseFillEmptyRowsGrad<T> extends PrimitiveOp {
  
  /**
   * Factory method to create a class to wrap a new SparseFillEmptyRowsGrad operation to the graph.
   * 
   * @param scope current graph scope
   * @param reverseIndexMap 1-D.  The reverse index map from SparseFillEmptyRows.
   * @param gradValues 1-D.  The gradients from backprop.
   * @return a new instance of SparseFillEmptyRowsGrad
   */
  public static <T> SparseFillEmptyRowsGrad<T> create(Scope scope, Operand<Long> reverseIndexMap, Operand<T> gradValues) {
    OperationBuilder opBuilder = scope.graph().opBuilder("SparseFillEmptyRowsGrad", scope.makeOpName("SparseFillEmptyRowsGrad"));
    opBuilder.addInput(reverseIndexMap.asOutput());
    opBuilder.addInput(gradValues.asOutput());
    return new SparseFillEmptyRowsGrad<T>(opBuilder.build());
  }
  
  /**
   * 1-D.  The backprop into values.
   */
  public Output<T> dValues() {
    return dValues;
  }
  
  /**
   * 0-D.  The backprop into default_value.
   */
  public Output<T> dDefaultValue() {
    return dDefaultValue;
  }
  
  private Output<T> dValues;
  private Output<T> dDefaultValue;
  
  private SparseFillEmptyRowsGrad(Operation operation) {
    super(operation);
    int outputIdx = 0;
    dValues = operation.output(outputIdx++);
    dDefaultValue = operation.output(outputIdx++);
  }
}
