/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the sum along sparse segments of a tensor.
 * <p>
 * Read @{$math_ops#Segmentation$the section on segmentation} for an explanation of
 * segments.
 * <p>
 * Like {@code SegmentSum}, but {@code segment_ids} can have rank less than {@code data}'s first
 * dimension, selecting a subset of dimension 0, specified by {@code indices}.
 * <p>
 * For example:
 * <pre>{@code
 * c = tf.constant([[1,2,3,4], [-1,-2,-3,-4], [5,6,7,8]])
 * 
 * # Select two rows, one segment.
 * tf.sparse_segment_sum(c, tf.constant([0, 1]), tf.constant([0, 0]))
 * # => [[0 0 0 0]]
 * 
 * # Select two rows, two segment.
 * tf.sparse_segment_sum(c, tf.constant([0, 1]), tf.constant([0, 1]))
 * # => [[ 1  2  3  4]
 * #     [-1 -2 -3 -4]]
 * 
 * # Select all rows, two segments.
 * tf.sparse_segment_sum(c, tf.constant([0, 1, 2]), tf.constant([0, 0, 1]))
 * # => [[0 0 0 0]
 * #     [5 6 7 8]]
 * 
 * # Which is equivalent to:
 * tf.segment_sum(c, tf.constant([0, 0, 1]))
 * }</pre>
 * 
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class SparseSegmentSum<T extends Number> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new SparseSegmentSum operation to the graph.
   * 
   * @param scope current graph scope
   * @param data 
   * @param indices A 1-D tensor. Has same rank as {@code segment_ids}.
   * @param segmentIds A 1-D tensor. Values should be sorted and can be repeated.
   * @return a new instance of SparseSegmentSum
   */
  public static <T extends Number, U extends Number> SparseSegmentSum<T> create(Scope scope, Operand<T> data, Operand<U> indices, Operand<Integer> segmentIds) {
    OperationBuilder opBuilder = scope.graph().opBuilder("SparseSegmentSum", scope.makeOpName("SparseSegmentSum"));
    opBuilder.addInput(data.asOutput());
    opBuilder.addInput(indices.asOutput());
    opBuilder.addInput(segmentIds.asOutput());
    return new SparseSegmentSum<T>(opBuilder.build());
  }
  
  /**
   * Has same shape as data, except for dimension 0 which
   * has size {@code k}, the number of segments.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private SparseSegmentSum(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
