/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Splits a tensor into {@code num_split} tensors along one dimension.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Split<T> extends PrimitiveOp implements Iterable<Operand<T>> {
  
  /**
   * Factory method to create a class to wrap a new Split operation to the graph.
   * 
   * @param scope current graph scope
   * @param axis 0-D.  The dimension along which to split.  Must be in the range
   * {@code [-rank(value), rank(value))}.
   * @param value The tensor to split.
   * @param numSplit The number of ways to split.  Must evenly divide
   * {@code value.shape[split_dim]}.
   * @return a new instance of Split
   */
  public static <T> Split<T> create(Scope scope, Operand<Integer> axis, Operand<T> value, Long numSplit) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Split", scope.makeOpName("Split"));
    opBuilder.addInput(axis.asOutput());
    opBuilder.addInput(value.asOutput());
    opBuilder.setAttr("num_split", numSplit);
    return new Split<T>(opBuilder.build());
  }
  
  /**
   * They are identically shaped tensors, whose shape matches that of {@code value}
   * except along {@code axis}, where their sizes are
   * {@code values.shape[split_dim] / num_split}.
   */
  public List<Output<T>> output() {
    return output;
  }
  
  @Override
  @SuppressWarnings({"rawtypes", "unchecked"})
  public Iterator<Operand<T>> iterator() {
    return (Iterator) output.iterator();
  }
  
  private List<Output<T>> output;
  
  @SuppressWarnings("unchecked")
  private Split(Operation operation) {
    super(operation);
    int outputIdx = 0;
    int outputLength = operation.outputListLength("output");
    output = Arrays.asList((Output<T>[])operation.outputList(outputIdx, outputLength));
    outputIdx += outputLength;
  }
}
