/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import java.util.List;
import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Converts each string in the input Tensor to its hash mod by a number of buckets.
 * <p>
 * The hash function is deterministic on the content of the string within the
 * process. The hash function is a keyed hash function, where attribute {@code key}
 * defines the key of the hash function. {@code key} is an array of 2 elements.
 * <p>
 * A strong hash is important when inputs may be malicious, e.g. URLs with
 * additional components. Adversaries could try to make their inputs hash to the
 * same bucket for a denial-of-service attack or to skew the results. A strong
 * hash prevents this by making it difficult, if not infeasible, to compute inputs
 * that hash to the same bucket. This comes at a cost of roughly 4x higher compute
 * time than {@code tf.string_to_hash_bucket_fast}.
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class StringToHashBucketStrong extends PrimitiveOp implements Operand<Long> {
  
  /**
   * Factory method to create a class to wrap a new StringToHashBucketStrong operation to the graph.
   * 
   * @param scope current graph scope
   * @param input The strings to assign a hash bucket.
   * @param numBuckets The number of buckets.
   * @param key The key for the keyed hash function passed as a list of two uint64
   * elements.
   * @return a new instance of StringToHashBucketStrong
   */
  public static StringToHashBucketStrong create(Scope scope, Operand<String> input, Long numBuckets, List<Long> key) {
    OperationBuilder opBuilder = scope.graph().opBuilder("StringToHashBucketStrong", scope.makeOpName("StringToHashBucketStrong"));
    opBuilder.addInput(input.asOutput());
    opBuilder.setAttr("num_buckets", numBuckets);
    long[] keyArray = new long[key.size()];
    for (int i = 0; i < keyArray.length; ++i) {
      keyArray[i] = key.get(i);
    }
    opBuilder.setAttr("key", keyArray);
    return new StringToHashBucketStrong(opBuilder.build());
  }
  
  /**
   * A Tensor of the same shape as the input {@code string_tensor}.
   */
  public Output<Long> output() {
    return output;
  }
  
  @Override
  public Output<Long> asOutput() {
    return output;
  }
  
  private Output<Long> output;
  
  private StringToHashBucketStrong(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
