/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the minimum along segments of a tensor.
 * <p>
 * Read @{$math_ops#segmentation$the section on segmentation} for an explanation of
 * segments.
 * <p>
 * This operator is similar to the unsorted segment sum operator found
 * [(here)](../../../api_docs/python/math_ops.md#UnsortedSegmentSum).
 * Instead of computing the sum over segments, it computes the minimum such that:
 * <p>
 * \\(output_i = \min_j data_j\\) where min is over {@code j} such
 * that {@code segment_ids[j] == i}.
 * <p>
 * If the minimum is empty for a given segment ID {@code i}, it outputs the largest
 * possible value for the specific numeric type,
 * {@code output[i] = numeric_limits<T>::max()}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class UnsortedSegmentMin<T extends Number> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new UnsortedSegmentMin operation to the graph.
   * 
   * @param scope current graph scope
   * @param data 
   * @param segmentIds A 1-D tensor whose rank is equal to the rank of {@code data}'s
   * first dimension.
   * @param numSegments 
   * @return a new instance of UnsortedSegmentMin
   */
  public static <T extends Number, U extends Number, V extends Number> UnsortedSegmentMin<T> create(Scope scope, Operand<T> data, Operand<U> segmentIds, Operand<V> numSegments) {
    OperationBuilder opBuilder = scope.graph().opBuilder("UnsortedSegmentMin", scope.makeOpName("UnsortedSegmentMin"));
    opBuilder.addInput(data.asOutput());
    opBuilder.addInput(segmentIds.asOutput());
    opBuilder.addInput(numSegments.asOutput());
    return new UnsortedSegmentMin<T>(opBuilder.build());
  }
  
  /**
   * Has same shape as data, except for dimension 0 which
   * has size {@code num_segments}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private UnsortedSegmentMin(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
