/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the sum along segments of a tensor.
 * <p>
 * Read @{$math_ops#Segmentation$the section on segmentation} for an explanation of
 * segments.
 * <p>
 * Computes a tensor such that
 * {@code (output[i] = sum_{j...} data[j...]} where the sum is over tuples {@code j...} such
 * that {@code segment_ids[j...] == i}.  Unlike {@code SegmentSum}, {@code segment_ids}
 * need not be sorted and need not cover all values in the full
 * range of valid values.
 * <p>
 * If the sum is empty for a given segment ID {@code i}, {@code output[i] = 0}.
 * If the given segment ID {@code i} is negative, the value is dropped and will not be
 * added to the sum of the segment.
 * <p>
 * {@code num_segments} should equal the number of distinct segment IDs.
 * <p>
 * <div style="width:70%; margin:auto; margin-bottom:10px; margin-top:20px;">
 * <img style="width:100%" src="https://www.tensorflow.org/images/UnsortedSegmentSum.png" alt>
 * </div>
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class UnsortedSegmentSum<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new UnsortedSegmentSum operation to the graph.
   * 
   * @param scope current graph scope
   * @param data 
   * @param segmentIds A tensor whose shape is a prefix of {@code data.shape}.
   * @param numSegments 
   * @return a new instance of UnsortedSegmentSum
   */
  public static <T, U extends Number, V extends Number> UnsortedSegmentSum<T> create(Scope scope, Operand<T> data, Operand<U> segmentIds, Operand<V> numSegments) {
    OperationBuilder opBuilder = scope.graph().opBuilder("UnsortedSegmentSum", scope.makeOpName("UnsortedSegmentSum"));
    opBuilder.addInput(data.asOutput());
    opBuilder.addInput(segmentIds.asOutput());
    opBuilder.addInput(numSegments.asOutput());
    return new UnsortedSegmentSum<T>(opBuilder.build());
  }
  
  /**
   * Has same shape as data, except for the first {@code segment_ids.rank}
   * dimensions, which are replaced with a single dimension which has size
   * {@code num_segments}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private UnsortedSegmentSum(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
