/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Selects elements from {@code x} or {@code y}, depending on {@code condition}.
 * <p>
 * The {@code x}, and {@code y} tensors must all have the same shape, and the
 * output will also have that shape.
 * <p>
 * The {@code condition} tensor must be a scalar if {@code x} and {@code y} are scalars.
 * If {@code x} and {@code y} are vectors or higher rank, then {@code condition} must be either a
 * scalar, a vector with size matching the first dimension of {@code x}, or must have
 * the same shape as {@code x}.
 * <p>
 * The {@code condition} tensor acts as a mask that chooses, based on the value at each
 * element, whether the corresponding element / row in the output should be
 * taken from {@code x} (if true) or {@code y} (if false).
 * <p>
 * If {@code condition} is a vector and {@code x} and {@code y} are higher rank matrices, then
 * it chooses which row (outer dimension) to copy from {@code x} and {@code y}.
 * If {@code condition} has the same shape as {@code x} and {@code y}, then it chooses which
 * element to copy from {@code x} and {@code y}.
 * <p>
 * For example:
 * <pre>{@code
 * # 'condition' tensor is [[True,  False]
 * #                        [False, True]]
 * # 't' is [[1, 2],
 * #         [3, 4]]
 * # 'e' is [[5, 6],
 * #         [7, 8]]
 * select(condition, t, e)  # => [[1, 6], [7, 4]]
 * 
 * 
 * # 'condition' tensor is [True, False]
 * # 't' is [[1, 2],
 * #         [3, 4]]
 * # 'e' is [[5, 6],
 * #         [7, 8]]
 * select(condition, t, e) ==> [[1, 2],
 *                              [7, 8]]
 * 
 * }</pre>
 * 
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Where3<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new Where3 operation to the graph.
   * 
   * @param scope current graph scope
   * @param condition 
   * @param x = A {@code Tensor} which may have the same shape as {@code condition}.
   * If {@code condition} is rank 1, {@code x} may have higher rank,
   * but its first dimension must match the size of {@code condition}.
   * @param y = A {@code Tensor} with the same type and shape as {@code x}.
   * @return a new instance of Where3
   */
  public static <T> Where3<T> create(Scope scope, Operand<Boolean> condition, Operand<T> x, Operand<T> y) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Select", scope.makeOpName("Where3"));
    opBuilder.addInput(condition.asOutput());
    opBuilder.addInput(x.asOutput());
    opBuilder.addInput(y.asOutput());
    return new Where3<T>(opBuilder.build());
  }
  
  /**
   * = A {@code Tensor} with the same type and shape as {@code x} and {@code y}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private Where3(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
